<?php
/**
 * @file
 * Entity controller class for PayEx Autopay agreements.
 */

class PayExAutopayAgreementController extends EntityAPIController {
  /**
   * Called when entity is created.
   */
  public function create(array $values = array()) {
    global $user;

    // Enforce sane values for created/changed timestamps.
    $values['created'] = gmdate('Y-m-d\Th:i:s', REQUEST_TIME);
    $values['changed'] = NULL;

    // Default values for these options.
    $values += array(
      'max_amount' => 10000,
      'purchase_operation' => 'SALE',
      'agreement_ref' => NULL,
      'description' => NULL,
    );

    return parent::create($values);
  }

  /**
   * Implements EntityAPIControllerInterface.
   */
  public function delete($ids, DatabaseTransaction $transaction = NULL) {
    $entities = $ids ? $this->load($ids) : FALSE;
    if (!$entities) {
      // Do nothing, in case invalid or no ids have been passed.
      return;
    }

    // If settings fail to load, bail out, without further action.
    if (!$settings = _payex_get_api_settings()) {
      return;
    }

    foreach ($entities as $agreement) {
      // If we don't have a proper agreement reference, it has likely 
      // not been registered with PayEx. In any case, 
      if (empty($agreement->agreement_ref)) {
        continue;
      }
      try {
        $pxa = new PxAgreement($settings['merchant_account'],  $settings['encryption_key'], $settings['live']);
        $response = $pxa->deleteAgreement(array('agreementRef' => $agreement->agreement_ref));
      }
      catch (Exception $e) {
        watchdog_exception('payex_autopay', $e);
        throw $e;
      }
    }

    return parent::delete($ids, $transaction);
  }

  /**
   * Implements EntityAPIControllerInterface.
   */
  public function save($agreement, DatabaseTransaction $transaction = NULL) {
    $transaction = isset($transaction) ? $transaction : db_transaction();

    // If owner information was not provided, default to the current user.
    if (!isset($agreement->owner_entity_type) || !isset($agreement->owner_entity_id)) {
      global $user;
      $agreement->owner_entity_id = $user->uid;
      $agreement->owner_entity_type = 'user';
    }

    // Make sure the changed timestamp is updated when the agreeement is 
    // changed after its first creation.
    $now = gmdate('Y-m-d\Th:i:s', REQUEST_TIME);
    if ($now != $agreement->created) {
      $agreement->changed = $now;
    }

    // When saving a new agreement, make sure we register it with PayEx.
    if (empty($agreement->agreement_id) || !empty($agreement->is_new)) {

      // If settings fail to load, bail out, without saving.
      if (!$settings = _payex_get_api_settings()) {
        return FALSE;
      }

      // Make sure the agreement is saved, before we create the 
      // transaction, so it has been assigned an ID.
      parent::save($agreement, $transaction);
      
      // The parameters to be sent to PayEx.
      $params = array(
        'merchantRef' => 'PayExAutopayAgreement:' . $agreement->agreement_id,
        'description' => $agreement->description,
        'purchaseOperation' => $agreement->purchase_operation,
        'maxAmount' => $agreement->max_amount,
      );

      try {
        $pxo = new PxAgreement($settings['merchant_account'],  $settings['encryption_key'], $settings['live']);
        $response = $pxo->createAgreement($params);
      }
      catch (Exception $e) {
        $transaction->rollback();
        watchdog_exception('payex_autopay', $e);
        throw $e;
      }

      // If we got a proper agreement ref, save it.
      if ($response) {
        $agreement->agreement_ref = $response;
        return parent::save($agreement, $transaction);
      }

      // In case of failure, return FALSE;
      return FALSE;

    }
    // For updates, just save the record normally.
    else {
      return parent::save($agreement, $transaction);
    }
  }
}
